@@ -14,6 +14,7 @@
 # https://redfish.dmtf.org/schemas/VirtualMedia.v1_2_0.json
 
 from http import client as http_client
+import logging
 
 from sushy import exceptions
 from sushy.resources import base
@@ -22,6 +23,8 @@ from sushy.resources import common
 from sushy.resources.manager import constants as mgr_cons
 from sushy import utils
 
+LOG = logging.getLogger(__name__)
+
 
 class ActionsField(base.CompositeField):
 
@@ -106,6 +109,71 @@ class VirtualMedia(base.ResourceBase):
             eject_uri = eject_media.target_uri
         return eject_uri, use_patch
 
+    def is_transfer_protocol_required(self, error=None):
+        """Check the response code and body and in case of failure
+
+        Try to determine if it happened due to missing TransferProtocolType.
+        """
+        if (error.code.endswith('GeneralError')
+           and 'TransferProtocolType' in error.detail):
+            return True
+
+        return (
+            (error.code.endswith(".ActionParameterMissing")
+             or error.code.endswith(".PropertyMissing"))
+            and (("#/TransferProtocolType" in error.related_properties)
+                 or ("/TransferProtocolType" in error.related_properties))
+        )
+
+    def is_transfer_method_required(self, error=None):
+        """Check the response code and body and in case of failure
+
+        Try to determine if it happened due to missing TransferMethod
+        """
+        if (error.code.endswith('GeneralError')
+                and 'TransferMethod' in error.detail):
+            return True
+        return False
+
+    def is_credentials_required(self, error=None):
+        """Check the response code and body and in case of failure
+
+        Try to determine if it happened due to missing Credentials
+        """
+
+        # Accept broader variants seen on NVIDIA DGX where "code" may be absent
+        try:
+            body = error.response.json()
+        except Exception:
+            body = {}
+
+        # Structured path: @Message.ExtendedInfo entries
+        ext = (body.get('error', {}) or {}).get('@Message.ExtendedInfo',
+                                                []) or []
+        for m in ext:
+            mid = (m.get('MessageId') or '')
+            msg = (m.get('Message') or '')
+            if mid.endswith('ActionParameterMissing') and (
+                'UserName' in msg or 'Password' in msg):
+                return True
+
+        # Legacy/structured: error.code + message mentions UserName/Password
+        err_obj = (body.get('error') or {})
+        code = (err_obj.get('code') or '')
+        msg = (err_obj.get('message') or '')
+        if code.endswith('GeneralError') and (
+            'UserName' in msg or 'Password' in msg
+                or 'parameter missing' in msg):
+            return True
+
+        # Text fallback: common strings observed in the field
+        raw_msg = ((body.get('error') or {}).get('message') or '') + ' ' + (
+            error.detail or '')
+        if ('requires the parameter UserName' in raw_msg
+                or 'requires the parameter Password' in raw_msg):
+            return True
+        return False
+
     def insert_media(self, image, inserted=True, write_protected=True,
                      username=None, password=None, transfer_method=None):
         """Attach remote media to virtual media
@@ -161,7 +229,39 @@ class VirtualMedia(base.ResourceBase):
                 payload['Inserted'] = False
             if not write_protected:
                 payload['WriteProtected'] = False
-            self._conn.post(target_uri, data=payload)
+            # Called a Max of 4 times, 1 plain + 1 for each of the 3 errors
+            for _ in range(4):
+                try:
+                    self._conn.post(target_uri, data=payload)
+                    break
+                except exceptions.HTTPError as error:
+                    # NOTE(janders) attempting to detect whether attachment
+                    # failure is due to absence of TransferProtocolType param
+                    # and if so adding it
+                    if payload.get('TransferProtocolType') is None and \
+                            self.is_transfer_protocol_required(error):
+                        if payload['Image'].startswith('https://'):
+                            payload['TransferProtocolType'] = "HTTPS"
+                        elif payload['Image'].startswith('http://'):
+                            payload['TransferProtocolType'] = "HTTP"
+                        continue
+
+                    # NOTE (iurygregory) we try to handle the case where a
+                    # a TransferMethod is also required in the payload.
+                    if payload.get('TransferMethod') != "Stream" and \
+                            self.is_transfer_method_required(error):
+                        payload['TransferMethod'] = "Stream"
+                        continue
+
+                    # NOTE (derekh) we try to handle the case where
+                    # credentials are required in the payload.
+                    # Seen on nvidia dgx (Bug 2071945)
+                    if payload.get('UserName') is None and \
+                            self.is_credentials_required(error):
+                        payload['UserName'] = "none"
+                        payload['Password'] = "none"  # noqa:S105
+                        continue
+                    raise
         self.invalidate()
 
     def eject_media(self):
@@ -190,7 +290,25 @@ class VirtualMedia(base.ResourceBase):
             if response.status_code in (
                     http_client.UNSUPPORTED_MEDIA_TYPE,
                     http_client.BAD_REQUEST):
-                self._conn.post(target_uri, data={})
+                try:
+                    self._conn.post(target_uri, data={})
+                except exceptions.HTTPError as vmd_exc:
+                    if (vmd_exc.status_code >= 500
+                        or vmd_exc.status_code in {http_client.BAD_REQUEST,
+                                                   http_client.CONFLICT}):
+                        LOG.warning('Received exception "%(exception)s" '
+                                    'while attempting to eject virtual media:'
+                                    ' %(id)s. Refreshing the resource.',
+                                    {'exception': vmd_exc,
+                                     'id': self.identity})
+                        self.refresh(force=True)
+                        if not self.inserted:
+                            LOG.debug("No image inserted on Virtual Media %s",
+                                      self.identity)
+                            pass
+                        else:
+                            raise
+
         self.invalidate()
 
     def set_verify_certificate(self, verify_certificate):
@@ -200,8 +318,10 @@ class VirtualMedia(base.ResourceBase):
                 parameter='verify_certificate', value=verify_certificate,
                 valid_values='boolean (True, False)')
 
+        etag = self._get_etag()
         self._conn.patch(self.path,
-                         data={'VerifyCertificate': verify_certificate})
+                         data={'VerifyCertificate': verify_certificate},
+                         etag=etag)
         self.invalidate()
 
     @property
